import enet

SHUTDOWN_MSG = "SHUTDOWN"

host = enet.Host(enet.Address(b"localhost", 54301), 10, 0, 0, 0)

connect_count = 0
run = True
shutdown_recv = False
while run:
    # Wait 1 second for an event
    event = host.service(1000)
    if event.type == enet.EVENT_TYPE_CONNECT:
        print("%s: CONNECT" % event.peer.address)
        connect_count += 1
    elif event.type == enet.EVENT_TYPE_DISCONNECT:
        print("%s: DISCONNECT" % event.peer.address)
        connect_count -= 1
        if connect_count <= 0 and shutdown_recv:
            run = False
    elif event.type == enet.EVENT_TYPE_RECEIVE:
        print("%s: IN:  %r" % (event.peer.address, event.packet.data))
        msg = event.packet.data
        if event.peer.send(0, enet.Packet(msg)) < 0:
            print("%s: Error sending echo packet!" % event.peer.address)
        else:
            print("%s: OUT: %r" % (event.peer.address, msg))
        if event.packet.data == b"SHUTDOWN":
            shutdown_recv = True

# Part of the test to do with intercept callback and socket.send

connect_count = 0
run = True
shutdown_recv = False

def receive_callback(address, data):
    if data and data == b"\xff\xff\xff\xffgetstatus\x00":
        host.socket.send(address, b"\xff\xff\xff\xffstatusResponse\n")

host.intercept = receive_callback

while run:
    # Wait 1 second for an event
    event = host.service(1000)
    if event.type == enet.EVENT_TYPE_CONNECT:
        print("%s: CONNECT" % event.peer.address)
        connect_count += 1
    elif event.type == enet.EVENT_TYPE_DISCONNECT:
        print("%s: DISCONNECT" % event.peer.address)
        connect_count -= 1
        if connect_count <= 0 and shutdown_recv:
            run = False
    elif event.type == enet.EVENT_TYPE_RECEIVE:
        print("%s: IN:  %r" % (event.peer.address, event.packet.data))

        # This packet echo is used to mimick the usual use case when packets are going back&forth while the intercept callback is used
        msg = event.packet.data
        if event.peer.send(0, enet.Packet(msg)) < 0:
            print("%s: Error sending echo packet!" % event.peer.address)
        else:
            print("%s: OUT: %r" % (event.peer.address, msg))
        if event.packet.data == b"SHUTDOWN":
            shutdown_recv = True



import unittest
import enet

class TestAddress(unittest.TestCase):
    def test_host(self):
        self.assertEquals(enet.Address("127.0.0.1", 9999).host, "127.0.0.1")
        self.assertEquals(enet.Address("localhost", 9999).host, "127.0.0.1")
        self.assertEquals(enet.Address(None, 9999).host, "*")
        self.assertRaises(IOError, enet.Address, "foo.bar.baz.999", 9999)

    def test_port(self):
        self.assertEquals(enet.Address("127.0.0.1", 9999).port, 9999)
        self.assertRaises(TypeError, enet.Address, "127.0.0.1", "foo")

    def test_hostname(self):
        import socket
        self.assertEquals(enet.Address(socket.gethostname(), 9999).hostname, socket.gethostname())
        self.assertEquals(enet.Address(None, 9999).hostname, "*")

    def test_str(self):
        self.assertEquals(enet.Address("127.0.0.1", 9999).__str__(), "127.0.0.1:9999")

    def test_richcmp(self):
        self.assertTrue(enet.Address("127.0.0.1", 9999) == enet.Address("127.0.0.1", 9999))
        self.assertTrue(enet.Address("127.0.0.1", 9999) != enet.Address("127.0.0.1", 8888))
        self.assertFalse(enet.Address("127.1.1.1", 1992) == enet.Address("127.0.0.1", 9999))

class TestPacket(unittest.TestCase):
    def test_data(self):
        self.assertEquals(enet.Packet(b"foo\0bar").data, b"foo\0bar")
        self.assertRaises(MemoryError, getattr, enet.Packet(), "data")

    def test_dataLength(self):
        self.assertEquals(enet.Packet(b"foobar").dataLength, 6)
        self.assertEquals(enet.Packet(b"foo\0bar").dataLength, 7)
        self.assertRaises(MemoryError, getattr, enet.Packet(), "dataLength")

    def test_flags(self):
        self.assertEquals(enet.Packet(b"foobar").flags, 0)
        self.assertEquals(enet.Packet(b"foobar", enet.PACKET_FLAG_UNSEQUENCED).flags, enet.PACKET_FLAG_UNSEQUENCED)
        self.assertRaises(MemoryError, getattr, enet.Packet(), "flags")

class TestHost(unittest.TestCase):
    def setUp(self):
        self.client = enet.Host(None, 1, 0, 0, 0)
        self.server = enet.Host(enet.Address("localhost", 54301), 1, 0, 0, 0)
        self.peer = self.client.connect(enet.Address("localhost", 54301), 1)
        self.assertEquals(self.peer.state, enet.PEER_STATE_CONNECTING)

    def tearDown(self):
        del self.client
        del self.server
        del self.peer

    def test_connect(self):
        client_connected = False
        server_connected = False

        counter = 0
        while counter < 100 or not (client_connected and server_connected):
            event = self.client.service(0)
            if event.type == enet.EVENT_TYPE_CONNECT:
                self.assertEquals(event.peer.state, enet.PEER_STATE_CONNECTED)
                client_connected = True
            event = self.server.service(0)
            if event.type == enet.EVENT_TYPE_CONNECT:
                self.assertEquals(event.peer.state, enet.PEER_STATE_CONNECTED)
                server_connected = True
            counter += 1

        self.assertEquals(client_connected, True)
        self.assertEquals(server_connected, True)

    def test_socketsend(self):

        self.send_done = False
        socketsend_msg = b"\xff\xff\xff\xffgetstatus\x00"

        def f(address, data):
            if data != socketsend_msg:
                # error messages are not propagating
                # through cython
                print("data != statusResponse")
                assert(False)
            self.send_done = True

        while not self.send_done:

            self.client.service(0)
            self.client.socket.send(self.server.address, socketsend_msg)

            event = self.server.service(0)
            if event.type == enet.EVENT_TYPE_CONNECT:
                self.server.intercept = f


    def test_broadcast(self):
        broadcast_done = False
        broadcast_msg = b"foo\0bar\n baz!"

        while not broadcast_done:
            event = self.client.service(0)
            if event.type == enet.EVENT_TYPE_RECEIVE:
                self.assertEquals(event.peer.state, enet.PEER_STATE_CONNECTED)
                self.assertEquals(event.packet.data, broadcast_msg)
                broadcast_done = True
            event = self.server.service(0)
            if event.type == enet.EVENT_TYPE_CONNECT:
                self.assertEquals(event.peer.state, enet.PEER_STATE_CONNECTED)
                self.server.broadcast(0, enet.Packet(broadcast_msg))

class TestPeer(unittest.TestCase):
    def setUp(self):
        self.client = enet.Host(None, 1, 0, 0, 0)
        self.server = enet.Host(enet.Address("localhost", 54301), 1, 0, 0, 0)
        self.peer = self.client.connect(enet.Address("localhost", 54301), 1)
        self.assertEquals(self.peer.state, enet.PEER_STATE_CONNECTING)
        self.assertTrue(self.peer == self.peer)
        self.assertFalse(self.peer != self.peer)

    def tearDown(self):
        del self.client
        del self.server
        del self.peer

    def test_access(self):
        self.assertRaises(MemoryError, enet.Peer().reset)

    def test_send(self):
        msg = b"foo\0bar"
        msg_received = False

        while not msg_received:
            event = self.server.service(0)
            if event.type == enet.EVENT_TYPE_RECEIVE:
                msg_received = True
                self.assertEquals(event.packet.data, msg)

            event = self.client.service(0)
            if event.type == enet.EVENT_TYPE_CONNECT:
                packet = enet.Packet(msg)
                self.assertEquals(packet.sent, False)
                ret = self.peer.send(0, packet)
                self.assertEquals(ret, 0)
                self.assertEquals(packet.sent, True)

    def test_reset(self):
        reset_done = False
        while not reset_done:
            event = self.server.service(0)
            if event.type == enet.EVENT_TYPE_CONNECT:
                self.assertEquals(event.peer.state, enet.PEER_STATE_CONNECTED)
                event.peer.reset()
                self.assertEquals(event.peer.state, enet.PEER_STATE_DISCONNECTED)
                reset_done = True
            event = self.client.service(0)

    def test_disconnect(self):
        connected = True
        while connected:
            event = self.server.service(0)
            if event.type == enet.EVENT_TYPE_DISCONNECT:
                self.assertEquals(event.peer.state, enet.PEER_STATE_DISCONNECTED)
                connected = False
            elif event.type == enet.EVENT_TYPE_CONNECT:
                self.assertEquals(self.peer.state, enet.PEER_STATE_CONNECTED)
                self.peer.disconnect()
                self.assertEquals(self.peer.state, enet.PEER_STATE_DISCONNECTING)

            event = self.client.service(0)
            if event.type == enet.EVENT_TYPE_CONNECT:
                self.assertEquals(event.peer.state, enet.PEER_STATE_CONNECTED)

    def test_disconnect_later(self):
        connected = True
        while connected:
            event = self.server.service(0)
            if event.type == enet.EVENT_TYPE_DISCONNECT:
                self.assertEquals(event.peer.state, enet.PEER_STATE_DISCONNECTED)
                connected = False
            elif event.type == enet.EVENT_TYPE_CONNECT:
                self.assertEquals(self.peer.state, enet.PEER_STATE_CONNECTED)
                self.peer.disconnect_later()
                self.assertEquals(self.peer.state, enet.PEER_STATE_DISCONNECT_LATER)

            event = self.client.service(0)
            if event.type == enet.EVENT_TYPE_CONNECT:
                self.assertEquals(event.peer.state, enet.PEER_STATE_CONNECTED)

if __name__ == '__main__':
    unittest.main()




from setuptools import setup
from distutils.extension import Extension
from distutils.core import run_setup

import os
import glob
import sys
import re

source_files = ["enet.pyx"]

_enet_files = glob.glob("enet/*.c")
lib_version = "1.3.13"
package_version = lib_version + '.post5'


try:
   import pypandoc
   long_description = pypandoc.convert_text(
        re.sub(r'[^\x00-\x7F]+', ' ',
            pypandoc.convert('README.md', 'markdown', format="markdown_github")), 'rst', format="markdown")
except (IOError, ImportError):
    print('pypandoc not available, using plain contents of readme')
    long_description = re.sub(r'[^\x00-\x7F]+', ' ', open('README.md').read())


from distutils.command.build_ext import build_ext as _build_ext
class build_ext(_build_ext):
    def run(self):

        from Cython.Build import cythonize
        self.extensions = cythonize(self.extensions)

        _build_ext.run(self)

        extra_args = sys.argv[2:]
        run_setup(os.path.join(os.getcwd(), "setup.py"), ['build_py'] + extra_args)


if not _enet_files:
    import tarfile
    try:
        import urllib.request as urllib_request
    except ImportError:
        import urllib as urllib_request

    enet_dir = "enet-%s/" % lib_version
    enet_file = "enet-%s.tar.gz" % lib_version
    enet_url = "http://enet.bespin.org/download/%s" % enet_file

    if not os.path.isfile(enet_file):
        print("Downloading enet")

        # begin code shamelessly stolen from StackOverflow
        # http://stackoverflow.com/a/13895723/2730399
        # Answer by https://stackoverflow.com/users/4279/j-f-sebastian
        def reporthook(blocknum, blocksize, totalsize):
            readsofar = blocknum * blocksize
            if totalsize > 0:
                percent = readsofar * 1e2 / totalsize
                s = "\r%5.1f%% %*d / %d" % (percent, len(str(totalsize)),
                                            readsofar, totalsize)
                sys.stderr.write(s)
                if readsofar >= totalsize:  # near the end
                    sys.stderr.write("\n")
            else:  # total size is unknown
                sys.stderr.write("read %d\n" % (readsofar, ))

        urllib_request.urlretrieve(enet_url, enet_file, reporthook)
        print("Finished downloading enet")
        # end code shamelessly stolen from StackOverflow

    print("Unpacking enet")
    tar = tarfile.open(enet_file)

    l = len(enet_dir)
    members = []
    for member in tar.getmembers():
        if member.path.startswith(enet_dir):
            member.path = member.path[l:]
            members.append(member)

    tar.extractall('enet', members=members)
    tar.close()
    print("Finished unpacking enet")

source_files.extend(_enet_files)

define_macros = [('HAS_POLL', None), ('HAS_FCNTL', None),
                 ('HAS_MSGHDR_FLAGS', None), ('HAS_SOCKLEN_T', None)]

libraries = []

if sys.platform == 'win32':
    define_macros.extend([('MS_WIN64', None)])
    libraries.extend(['enet64', 'Winmm', 'ws2_32'])

if sys.platform != 'darwin':
    define_macros.extend([('HAS_GETHOSTBYNAME_R', None),
                          ('HAS_GETHOSTBYADDR_R', None)])

ext_modules = [
    Extension(
        "enet",
        extra_compile_args=["-O3"],
        sources=source_files,
        include_dirs=["enet/include/"],
        define_macros=define_macros,
        libraries=libraries,
        library_dirs=["enet/"])
]

setup(
    name='pyenet',
    # packages=['enet'],
    description='A python wrapper for the ENet library',
    long_description=long_description,
    url='https://github.com/piqueserver/pyenet/',
    maintainer='Andrew Resch, Piqueserver team',
    maintainer_email='samuel@swalladge.id.au',

    version=package_version,
    cmdclass={'build_ext': build_ext},
    ext_modules=ext_modules,
    setup_requires=['Cython>=0,<1'],
    install_requires=['Cython>=0,<1'],
)
